#include "flash.h"
#include <string.h>

#define FLASH_PROG_ADDR         0x00004000
#define FLASH_PAGE_SIZE         256

#define BUFFER_SIZE     128
uint32_t DataBuffer[BUFFER_SIZE];

void FlashRW(void)
{
    FL_FLASH_PageErase(FLASH, FLASH_PROG_ADDR);
    memset(DataBuffer, 0x55, BUFFER_SIZE * 4);
    FL_FLASH_Program_Word(FLASH, FLASH_PROG_ADDR, DataBuffer[0]);

    FL_FLASH_PageErase(FLASH, FLASH_PROG_ADDR);
    memset(DataBuffer, 0xAA, BUFFER_SIZE * 4);
    FL_FLASH_Program_Page(FLASH, FLASH_PROG_ADDR / FL_FLASH_PAGE_SIZE_BYTE, DataBuffer);

    FL_FLASH_SectorErase(FLASH, FLASH_PROG_ADDR);
    memset(DataBuffer, 0x00, BUFFER_SIZE * 4);
    FL_FLASH_Program_Sector(FLASH, FLASH_PROG_ADDR / FL_FLASH_SECTOR_SIZE_BYTE, DataBuffer);

    FL_FLASH_SectorErase(FLASH, FLASH_PROG_ADDR);
}

extern FL_ErrorStatus FL_FLASH_Write_Dma(FLASH_Type *FLASHx, uint32_t address, uint32_t *data);
extern FL_ErrorStatus FL_FLASH_Read_Dma(FLASH_Type *FLASHx, uint32_t address, uint32_t *data, uint16_t length);

void FlashRW_Dma(void)
{
    
    FL_RCC_EnableGroup2BusClock(FL_RCC_GROUP2_BUSCLK_DMA);

    FL_DMA_Enable(DMA);
    
    FL_FLASH_PageErase(FLASH, FLASH_PROG_ADDR);
    memset(DataBuffer, 0x00, BUFFER_SIZE * 4);
    FL_FLASH_Read_Dma(FLASH, FLASH_PROG_ADDR, DataBuffer, FLASH_PAGE_SIZE / 4);
    memset(DataBuffer, 0x55, BUFFER_SIZE * 4);
    FL_FLASH_Write_Dma(FLASH, FLASH_PROG_ADDR, DataBuffer);
    memset(DataBuffer, 0x00, BUFFER_SIZE * 4);
    FL_FLASH_Read_Dma(FLASH, FLASH_PROG_ADDR, DataBuffer, FLASH_PAGE_SIZE / 4);

    FL_FLASH_PageErase(FLASH, FLASH_PROG_ADDR);
    memset(DataBuffer, 0x00, BUFFER_SIZE * 4);
    FL_FLASH_Read_Dma(FLASH, FLASH_PROG_ADDR, DataBuffer, FLASH_PAGE_SIZE / 4);
    memset(DataBuffer, 0xAA, BUFFER_SIZE * 4);
    FL_FLASH_Write_Dma(FLASH, FLASH_PROG_ADDR, DataBuffer);
    memset(DataBuffer, 0x00, BUFFER_SIZE * 4);
    FL_FLASH_Read_Dma(FLASH, FLASH_PROG_ADDR, DataBuffer, FLASH_PAGE_SIZE / 4);

    FL_FLASH_PageErase(FLASH, FLASH_PROG_ADDR);
    FL_DMA_Disable(DMA);
}

